#include <avr/io.h>
//#include <avr/interrupt.h>
#include <inttypes.h>
#include <util/delay.h>

#define RFM12TxBDW(kfrq)	((unsigned char)(kfrq/15)-1)
#define RFM12FREQ(freq)	((freq-430.0)/0.0025)	


void SetupRFM12(void)
{
	// PORTB 5 = SCK
	// PORTB 4 = SDO
	// PORTB 3 = SDI
	// PORTB 2 = nSEL
	// PORTD 2 = nIRQ
	DDRB |= (1<<DDB2) | (1<<DDB3) | (1<<DDB5);
	DDRB &= ~(1<<DDB4);
	DDRD &= ~(1<<DDD2);
	
	PORTB |= (1<<PB2);
	PORTB |= (1<<PB3);
	PORTB &= ~(1<<PB5);
}

unsigned int RFM12_write(unsigned int cmd)
{
	unsigned int temp = 0x0000;
	unsigned char i;
	PORTB &= ~(1<<PB5);
	PORTB &= ~(1<<PB2);
	for (i = 0; i < 16; i++)
	{
		temp <<= 1;
		if (PINB & (1<<PB4))
		{
			temp |= 0x0001;
		}
		PORTB &= ~(1<<PB5);
		if (cmd & 0x8000)
		{
			PORTB |= (1<<PB3);
		}
		else
		{
			PORTB &= ~(1<<PB3);
		}
		PORTB |= (1<<PB5);
		cmd <<= 1;
	}
	PORTB &= (1<<PB5);
	PORTB |= (1<<PB2);
	return temp;
}

void RFM12_setbandwidth(unsigned char bandwidth, unsigned char gain, 
						unsigned char drssi)
{
	RFM12_write(0x9400|((bandwidth&7)<<5)|((gain&3)<<3)|(drssi&7));
}

void RFM12_setfreq(unsigned short freq)
{	
	if (freq<96)					// 430,2400MHz
		freq=96;
	else if (freq>3903)			// 439,7575MHz
		freq=3903;
	RFM12_write(0xA000|freq);
}

void RFM12_setbaud(unsigned short baud)
{
	if (baud<663)
		return;
	if (baud<5400)					// Baudrate= 344827,58621/(R+1)/(1+CS*7)
		RFM12_write(0xC680|((43104/baud)-1));
	else
		RFM12_write(0xC600|((344828UL/baud)-1));
}

void RFM12_setpower(unsigned char power, unsigned char mod)
{	
	RFM12_write(0x9800|(power&7)|((mod&15)<<4));
}

void RFM12_send(unsigned char data)
{
	while (PIND & (1<<PD2)) asm volatile ("nop");
	RFM12_write(0xB800+data);
}

void RFM12_init(void)
{
	uint8_t i = 0;
	// AVR runs at RFM12 default CLK of 1MHz, Project is set up for 10MHz, so this will take about 1 sec, enough for POR
	for (i=0; i<10; i++)
		_delay_ms(10);			

	RFM12_write(0xC0E0);			// AVR CLK: 10MHz
	RFM12_write(0x80D7);			// Enable FIFO
	RFM12_write(0xC2AB);			// Data Filter: internal
	RFM12_write(0xCA81);			// Set FIFO mode
	RFM12_write(0xE000);			// disable wakeuptimer
	RFM12_write(0xC800);			// disable low duty cycle
	RFM12_write(0xC4F7);			// AFC settings: autotuning: -10kHz...+7,5kHz
	RFM12_write(0x0000);

	// AVR runs at RFM12 CLK of 10MHz now, so this will take about 100 ms
	for (i=0; i<10; i++)
		_delay_ms(10);			

	RFM12_setfreq(RFM12FREQ(433.92));	// Sende/Empfangsfrequenz auf 433,92MHz einstellen
	RFM12_setbandwidth(5, 1, 4);		// 200kHz Bandbreite, -6dB Verstrkung, DRSSI threshold: -79dBm 
	RFM12_setbaud(9600);				// 9600 baud
	RFM12_setpower(0, 6);			// 1mW Ausgangangsleistung, 120k
}

void blink(void)
{
	unsigned short c0, c1;
	unsigned short c0max, c1max;
	c0max = 255;
	c1max = 255;
	PORTC &= ~(1<<PC5);
	for(c0 = 0; c0 < c0max; c0++)
	{
		for(c1 = 0; c1 < c1max; c1++)
		{
			asm volatile ("nop");
		}
	}
	PORTC |= (1<<PC5);
	for(c0 = 0; c0 < c0max; c0++)
	{
		for(c1 = 0; c1 < c1max; c1++)
		{
			asm volatile ("nop");
		}
	}
}

int main(void)
{
	DDRC |= (1<<DDC5);
	PORTC |= (1<<PC5);

	/*uint8_t i = 0;
	for (i = 0; i < 10; i++)
	{
		blink();
	}
	PORTC |= (1<<PC5);*/

	SetupRFM12();
	RFM12_init();
	//sei();

	for (;;)
	{
		blink();
	}
	return 1;
}
